Backport a glibc-2.31 fix:

"""
From c261718b3fda4b629ee120f48e3ee50db9c793a1 Mon Sep 17 00:00:00 2001
From: Adhemerval Zanella <adhemerval.zanella@linaro.org>
Date: Mon, 10 Feb 2020 12:57:28 -0300
Subject: [PATCH] sparc: Avoid clobbering register parameters in syscall

The sparc INTERNAL_SYSCALL macro might clobber the register
parameter if the argument itself might clobber any register (a function
call for instance).

This patch fixes it by using temporary variables for the expressions
between the register assignments (as indicated by GCC documentation,
6.47.5.2 Specifying Registers for Local Variables).
"""
--- a/sysdeps/unix/sysv/linux/sparc/sysdep.h
+++ b/sysdeps/unix/sysv/linux/sparc/sysdep.h
@@ -88,7 +88,8 @@
 
 #define inline_syscall1(string,err,name,arg1)				\
 ({									\
-	register long __o0 __asm__ ("o0") = (long)(arg1);		\
+	long int _arg1 = (long int) (arg1);				\
+	register long __o0 __asm__ ("o0") = _arg1;			\
 	err = name;							\
 	__asm __volatile (string : "=r" (err), "=r" (__o0) :		\
 			  "0" (err), "1" (__o0) :			\
@@ -98,8 +99,10 @@
 
 #define inline_syscall2(string,err,name,arg1,arg2)			\
 ({									\
-	register long __o0 __asm__ ("o0") = (long)(arg1);		\
-	register long __o1 __asm__ ("o1") = (long)(arg2);		\
+	long int _arg1 = (long int) (arg1);				\
+	long int _arg2 = (long int) (arg2);				\
+	register long __o0 __asm__ ("o0") = _arg1;			\
+	register long __o1 __asm__ ("o1") = _arg2;			\
 	err = name;							\
 	__asm __volatile (string : "=r" (err), "=r" (__o0) :		\
 			  "0" (err), "1" (__o0), "r" (__o1) :		\
@@ -109,9 +112,12 @@
 
 #define inline_syscall3(string,err,name,arg1,arg2,arg3)			\
 ({									\
-	register long __o0 __asm__ ("o0") = (long)(arg1);		\
-	register long __o1 __asm__ ("o1") = (long)(arg2);		\
-	register long __o2 __asm__ ("o2") = (long)(arg3);		\
+	long int _arg1 = (long int) (arg1);				\
+	long int _arg2 = (long int) (arg2);				\
+	long int _arg3 = (long int) (arg3);				\
+	register long __o0 __asm__ ("o0") = _arg1;			\
+	register long __o1 __asm__ ("o1") = _arg2;			\
+	register long __o2 __asm__ ("o2") = _arg3;			\
 	err = name;							\
 	__asm __volatile (string : "=r" (err), "=r" (__o0) :		\
 			  "0" (err), "1" (__o0), "r" (__o1),		\
@@ -122,10 +128,14 @@
 
 #define inline_syscall4(string,err,name,arg1,arg2,arg3,arg4)		\
 ({									\
-	register long __o0 __asm__ ("o0") = (long)(arg1);		\
-	register long __o1 __asm__ ("o1") = (long)(arg2);		\
-	register long __o2 __asm__ ("o2") = (long)(arg3);		\
-	register long __o3 __asm__ ("o3") = (long)(arg4);		\
+	long int _arg1 = (long int) (arg1);				\
+	long int _arg2 = (long int) (arg2);				\
+	long int _arg3 = (long int) (arg3);				\
+	long int _arg4 = (long int) (arg4);				\
+	register long __o0 __asm__ ("o0") = _arg1;			\
+	register long __o1 __asm__ ("o1") = _arg2;			\
+	register long __o2 __asm__ ("o2") = _arg3;			\
+	register long __o3 __asm__ ("o3") = _arg4;			\
 	err = name;							\
 	__asm __volatile (string : "=r" (err), "=r" (__o0) :		\
 			  "0" (err), "1" (__o0), "r" (__o1),		\
@@ -136,11 +146,16 @@
 
 #define inline_syscall5(string,err,name,arg1,arg2,arg3,arg4,arg5)	\
 ({									\
-	register long __o0 __asm__ ("o0") = (long)(arg1);		\
-	register long __o1 __asm__ ("o1") = (long)(arg2);		\
-	register long __o2 __asm__ ("o2") = (long)(arg3);		\
-	register long __o3 __asm__ ("o3") = (long)(arg4);		\
-	register long __o4 __asm__ ("o4") = (long)(arg5);		\
+	long int _arg1 = (long int) (arg1);				\
+	long int _arg2 = (long int) (arg2);				\
+	long int _arg3 = (long int) (arg3);				\
+	long int _arg4 = (long int) (arg4);				\
+	long int _arg5 = (long int) (arg5);				\
+	register long __o0 __asm__ ("o0") = _arg1;			\
+	register long __o1 __asm__ ("o1") = _arg2;			\
+	register long __o2 __asm__ ("o2") = _arg3;			\
+	register long __o3 __asm__ ("o3") = _arg4;			\
+	register long __o4 __asm__ ("o4") = _arg5;			\
 	err = name;							\
 	__asm __volatile (string : "=r" (err), "=r" (__o0) :		\
 			  "0" (err), "1" (__o0), "r" (__o1),		\
@@ -151,12 +166,18 @@
 
 #define inline_syscall6(string,err,name,arg1,arg2,arg3,arg4,arg5,arg6)	\
 ({									\
-	register long __o0 __asm__ ("o0") = (long)(arg1);		\
-	register long __o1 __asm__ ("o1") = (long)(arg2);		\
-	register long __o2 __asm__ ("o2") = (long)(arg3);		\
-	register long __o3 __asm__ ("o3") = (long)(arg4);		\
-	register long __o4 __asm__ ("o4") = (long)(arg5);		\
-	register long __o5 __asm__ ("o5") = (long)(arg6);		\
+	long int _arg1 = (long int) (arg1);				\
+	long int _arg2 = (long int) (arg2);				\
+	long int _arg3 = (long int) (arg3);				\
+	long int _arg4 = (long int) (arg4);				\
+	long int _arg5 = (long int) (arg5);				\
+	long int _arg6 = (long int) (arg6);				\
+	register long __o0 __asm__ ("o0") = _arg1;			\
+	register long __o1 __asm__ ("o1") = _arg2;			\
+	register long __o2 __asm__ ("o2") = _arg3;			\
+	register long __o3 __asm__ ("o3") = _arg4;			\
+	register long __o4 __asm__ ("o4") = _arg5;			\
+	register long __o5 __asm__ ("o5") = _arg6;			\
 	err = name;							\
 	__asm __volatile (string : "=r" (err), "=r" (__o0) :		\
 			  "0" (err), "1" (__o0), "r" (__o1),		\
@@ -168,11 +189,16 @@
 
 #define INLINE_CLONE_SYSCALL(arg1,arg2,arg3,arg4,arg5)			\
 ({									\
-	register long __o0 __asm__ ("o0") = (long)(arg1);		\
-	register long __o1 __asm__ ("o1") = (long)(arg2);		\
-	register long __o2 __asm__ ("o2") = (long)(arg3);		\
-	register long __o3 __asm__ ("o3") = (long)(arg4);		\
-	register long __o4 __asm__ ("o4") = (long)(arg5);		\
+	long int _arg1 = (long int) (arg1);				\
+	long int _arg2 = (long int) (arg2);				\
+	long int _arg3 = (long int) (arg3);				\
+	long int _arg4 = (long int) (arg4);				\
+	long int _arg5 = (long int) (arg5);				\
+	register long __o0 __asm__ ("o0") = _arg1;			\
+	register long __o1 __asm__ ("o1") = _arg2;			\
+	register long __o2 __asm__ ("o2") = _arg3;			\
+	register long __o3 __asm__ ("o3") = _arg4;			\
+	register long __o4 __asm__ ("o4") = _arg5;			\
 	register long __g1 __asm__ ("g1") = __NR_clone;			\
 	__asm __volatile (__SYSCALL_STRING :				\
 			  "=r" (__g1), "=r" (__o0), "=r" (__o1)	:	\
